/******************************************************************************
 *	File	:	EcmUsrDriver.c
 *	Version :	1.1
 *	Date	:	2021/01/26
 *	Author	:	XFORCE
 *
 *	ECM-XF basic driver example - Source file
 *
 *	Demonstrate how to implement API type user driver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/

#include "EcmUsrDriver.h"
#include "platform.h"


uint8_t u8TxBuf[PKG_MAX_SIZE]={0};
uint8_t u8RxBuf[PKG_MAX_SIZE]={0};

SPI_CMD_PACKAGE_T *pCmd=(SPI_CMD_PACKAGE_T *)u8TxBuf;
SPI_RET_PACKAGE_T *pRet=(SPI_RET_PACKAGE_T *)u8RxBuf;

#define SPI_CRC_ERROR_CONTI_MAX	10

uint8_t g_u8CrcErrConti;
uint8_t g_u8EcmStatus;
uint8_t g_u8RxPDOFifoCnt;
uint8_t g_u8TxPDOFifoCnt;
uint8_t g_u8CrcErrCnt;
uint8_t g_u8WkcErrCnt;

int ECM_GetRetStatus(uint8_t *pStatus)
{
	*pStatus = pRet->Head.u8Status;
	return 1;
}
int ECM_GetRetErrStatus(uint8_t *pErrStatus)
{
	*pErrStatus = pRet->Head.u8ErrorStatus;
	return 1;
}
/*
 * Desc
 * 		Show error status and set clean flag
 * Param
 * 		nCleanFlag
 * 			0 : Don't clear error
 * 			1 : Clear error
 * Return
 * 		1 : No error
 * 		-1: Error occur
 * */
int ECM_PrintErrStatus(int nCleanFlag)
{
	if(pRet->Head.u8ErrorStatus & 0xF8){
		if(pRet->Head.u8ErrorStatus & ECM_ERR_ASYNC_CMD_MASK){
			printf("=====\r\n(%d)Cmd Error!!! - ASYNC CMD\r\n=====\r\n", pRet->Head.u8Cmd);
		}
		if(pRet->Head.u8ErrorStatus & ECM_ERR_SPI_MASK){
			printf("=====\r\n(%d)Cmd Error!!! - SPI\r\n=====\r\n", pRet->Head.u8Cmd);
		}
		if(pRet->Head.u8ErrorStatus & ECM_ERR_FIFO_MASK){
			printf("=====\r\n(%d)Cmd Error!!! - FIFO\r\n=====\r\n", pRet->Head.u8Cmd);
		}
		if(pRet->Head.u8ErrorStatus & ECM_ERR_CRC_MASK){
			printf("=====\r\n(%d)Cmd Error!!! - CRC\r\n=====\r\n", pRet->Head.u8Cmd);
		}
		if(pRet->Head.u8ErrorStatus & ECM_ERR_CMD_MASK){
			printf("=====\r\n(%d)Cmd Error!!! - CMD\r\n=====\r\n", pRet->Head.u8Cmd);
		}
		if(nCleanFlag){
			pCmd->Head.u8Ctrl |= ECM_CTL_CLR_ALL_ERR_MASK;
		}else{
			pCmd->Head.u8Ctrl &= ~ECM_CTL_CLR_ALL_ERR_MASK;
		}
		printf("Enter any key to continue\r\n");
		getchar();
		return -1;
	}else{
		pCmd->Head.u8Ctrl &= ~ECM_CTL_CLR_ALL_ERR_MASK;
		return 1;
	}
}
int EcmSpiPackSizeCal(int nSizeLimit)
{
	int i,j;
	int nStart=-1,nEnd=-1;
	int nSize=0;
	uint8_t u8Word[8];
	if(nSizeLimit<32){
		return -1;
	}
	if(nSizeLimit>PKG_MAX_SIZE){
		return -2;
	}
	for(i=0; i<nSizeLimit; i++){
		UserSpiDataExchange(u8TxBuf, u8RxBuf, 1);
		for(j=7;j>0;j--){
			u8Word[j] = u8Word[j-1];
		}
		u8Word[0] = u8RxBuf[0];
		//Start Word
		if( u8Word[7]==0x59 && \
			u8Word[6]==0x58 && \
			u8Word[5]==0x57 && \
			u8Word[4]==0x56 && \
			u8Word[3]==0xA4 && \
			u8Word[2]==0xA3 && \
			u8Word[1]==0xA2 && \
			u8Word[0]==0xA1){
			if(nStart < 0){
				nStart = (i-3);
			}else{
				nEnd = (i-4);
				nSize = nEnd-nStart+1;
				break;
			}
		}
		//Stop Word
		if(u8Word[0]==0x56 && u8Word[1]==0x57 && u8Word[2]==0x58 && u8Word[3]==0x59){
			UserDelay(200);
		}
	}
	if(nSize){
		// Already sent 4bytes start word, so do the rest bytes
		UserSpiDataExchange(u8TxBuf, u8RxBuf, nSize-4);
		UserDelay(200);
	}
	printf("SPI total package size %d\r\n",nSize);
	return nSize;
}
/*
 * Return
 * 			0: CRC error
 * 			1: Successful
 * */
int SpiDataExchange(uint8_t *RetIdx, uint8_t *RetCmd)
{
	int i=0;
	if(g_u8CrcErrConti >= SPI_CRC_ERROR_CONTI_MAX){
		printf("g_u8CrcErrConti reached MAX(%d)\r\n",SPI_CRC_ERROR_CONTI_MAX);
		return -1;
	}
	pCmd->Head.u32StartWord = ECM_START_WORD;
	pCmd->Crc = ECM_CRC_MAGIC_NUM;
	pCmd->StopWord = ECM_STOP_WORD;
	UserSpiDataExchange(u8TxBuf, u8RxBuf, sizeof(SPI_CMD_PACKAGE_T));
	UserDelay(TEST_SPI_IDLE_TIME);
	if(pRet->Crc == ECM_CRC_MAGIC_NUM){
		g_u8EcmStatus = pRet->Head.u8Status;
		g_u8RxPDOFifoCnt  = pRet->Head.u8RxFifoCnt;
		g_u8TxPDOFifoCnt  = pRet->Head.u8TxFifoCnt;
		g_u8CrcErrCnt = pRet->Head.u8CrcErrCnt;
		g_u8WkcErrCnt = pRet->Head.u8WkcErrCnt;
		if(RetIdx){
			*RetIdx = pRet->Head.u8Idx;
		}
		if(RetCmd){
			*RetCmd = pRet->Head.u8Cmd;
		}
		g_u8CrcErrConti=0;
		return 1;
	}
	// CRC Check Fail.
	for(i=0; i<sizeof(SPI_CMD_PACKAGE_T)-3; i++){
		if(u8RxBuf[i]==0x59 && u8RxBuf[i+1]==0x58 && u8RxBuf[i+2]==0x57 && u8RxBuf[i+3]==0x56){
			UserDelay(TEST_SPI_IDLE_TIME);
			break;
		}
	}
	if (i>=sizeof(SPI_CMD_PACKAGE_T)-4){
		printf("CRC Error(Idx : %d) - (Cmd : %d)\r\n",pCmd->Head.u8Idx,pCmd->Head.u8Cmd);
	}else{
		printf("shift (%d)\r\n",sizeof(SPI_CMD_PACKAGE_T)-i-4);
		UserSpiDataExchange(u8TxBuf, u8RxBuf, i+4);
		UserDelay(2000);//2ms
	}
	g_u8CrcErrConti++;
	return 0;
}
/*
 * Return
 * 			0	:	Timeout
 * 			1	:	Get valid return
 * */
int EcmCmdTransceiver(int nTryCnt)
{
	int i;
	uint8_t IdxCheck, CmdCheck;
	for(i=0;i<nTryCnt;i++){
		if(SpiDataExchange(&IdxCheck, &CmdCheck)){
			if(pCmd->Head.u8Idx == IdxCheck){
				if(pCmd->Head.u8Cmd == CmdCheck){
					return ECM_PrintErrStatus(1);
				}else{
					printf("Command code error %d != %d\r\n",pCmd->Head.u8Cmd ,CmdCheck);
					return 0;
				}
			}
		}
	}
	return 0;
}
/*	Return
 * 			0	:	Timeout
 * 			1	:	Get valid return
 * 			-1	:	Get valid return but command response error
 *			-2	:	WaitAyncDone error
 *			-3	:	Param error
 * */
int EcmCmdTransceiver_WaitAyncDone(int nTryCnt, uint16_t *pu16ReturnDataSize, uint8_t *pu8ReturnData)
{
	int nret=0;
	nret=EcmCmdTransceiver(nTryCnt);
	if(nret == 1){
		if(pu8ReturnData){
			memcpy(pu8ReturnData,pRet->Data,pRet->Head.u16Size);
			if(pu16ReturnDataSize){
				*pu16ReturnDataSize = pRet->Head.u16Size;
				return 1;//If the command want to get data back, so it's not an AsyncCommand. return immediately
			}else{
				return -3;
			}
		}
		if(ECM_WaitAsyncDone(nTryCnt)){
			return ECM_PrintErrStatus(1);//Show error message and set clean flag
		}else{
			nret = -2;
		}
	}
	return nret;
}
/*	Return
 * 			0	:	Timeout
 * 			1	:	Get valid return
 * 			-1	:	Get valid return but command response error
 *			-2	:	WaitAyncDone error *
 * */
int EcmUniversalCmd(	uint8_t u8Cmd, \
				uint8_t u8Param, \
				uint8_t u8Param0, \
				uint8_t u8Param1, \
				uint8_t u8Param2, \
				uint8_t u8Param3, \
				uint8_t *pu8Return, \
				uint8_t *pu8Status, \
				uint8_t *pu8ErrStatus, \
				uint16_t *pu16ReturnDataSize, \
				uint8_t *pu8ReturnData)
{

	int nret=0;
	pCmd->Head.u8Cmd = u8Cmd;
	pCmd->Head.u8Param = u8Param;
	pCmd->Head.u8Data[0] = u8Param0;
	pCmd->Head.u8Data[1] = u8Param1;
	pCmd->Head.u8Data[2] = u8Param2;
	pCmd->Head.u8Data[3] = u8Param3;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret=EcmCmdTransceiver_WaitAyncDone(1000, pu16ReturnDataSize, pu8ReturnData);
	if(nret > 0){
		if(pu8Return){
			*pu8Return = pRet->Head.u8Return;
		}
		if(pu8Status){
			*pu8Status = pRet->Head.u8Status;
		}
		if(pu8ErrStatus){
			*pu8ErrStatus = pRet->Head.u8ErrorStatus;
		}
	}
	return nret;
}
/*
 * Return
 * 0	:	CRC Error
 * 1	:	Get valid info
 * */
int ECM_InfoUpdate(uint8_t *pEcmStatus, uint8_t *pRxPDOFifoCnt, uint8_t *CrcErrCnt, uint8_t *WkcErrCnt)
{
	pCmd->Head.u8Cmd = ECM_CMD_INFO_UPDATE_OP;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	//Don't update ec_state, if update ec_state that will be an async command
	pCmd->Head.u8Ctrl &= ~ECM_CTL_UPDATE_EC_STATE_MASK;
	if(SpiDataExchange(0,0)){
		if(pRet->Head.u8ErrorStatus & 0xF8){
			printf("Cmd(%d) Error(0x%X)\r\n",pRet->Head.u8Cmd,pRet->Head.u8ErrorStatus);
		}
		if(pEcmStatus)
			*pEcmStatus = pRet->Head.u8Status;
		if(pRxPDOFifoCnt)
			*pRxPDOFifoCnt  = pRet->Head.u8RxFifoCnt;
		if(CrcErrCnt)
			*CrcErrCnt = pRet->Head.u8CrcErrCnt;
		if(WkcErrCnt)
			*WkcErrCnt = pRet->Head.u8WkcErrCnt;
		return 1;
	}
	return 0;
}
/*
 * Return
 * 	0		:	Error
 * 	Others	:	ECM firmware version
 *
 * */
uint8_t ECM_InitLibrary(uint16_t *pu16SpiDataSize)
{
	int i=0,j=0;
	int nret;
	uint8_t u8Version;
	memset(u8TxBuf,0,PKG_MAX_SIZE);
	memset(u8RxBuf,0,PKG_MAX_SIZE);
	pCmd=(SPI_CMD_PACKAGE_T *)u8TxBuf;
	pRet=(SPI_RET_PACKAGE_T *)u8RxBuf;
	pCmd->Head.u8Ctrl = ECM_CTL_CLR_ALL_ERR_MASK;
	g_u8CrcErrConti=0;
	g_u8TxPDOFifoCnt=0;

	if(pu16SpiDataSize){
		nret = ECM_SetSpiDataSize(*pu16SpiDataSize);
		if(nret <= 0){
			printf("ECM_SetSpiDataSize failed\r\n");
			return 0;
		}
		nret = ECM_GetSpiDataSize(pu16SpiDataSize);
		if(nret <= 0){
			printf("ECM_GetSpiDataSize failed\r\n");
			return 0;
		}
	}
	nret = ECM_GetFirmwareVersion(&u8Version);
	if(nret == 0){
		printf("ECM_GetFirmwareVersion failed\r\n");
		return 0;
	}
	for(i=0;i<10;i++){
		if(ECM_InfoUpdate(0, 0, 0, 0)){
			printf("Check ASYNC command");
			// FIFO has been initiated
			while(pRet->Head.u8Status & ECM_STA_ASYNC_OP_BUSY_MASK){
				if(j%10 == 9)
					printf("\rCheck ASYNC command");
				else
					printf(".");
				UserDelay(1000);
				if(j>=1000){
					printf("\r\nWait ASYNC done timeout\r\n");
					return 0;
				}
				j++;
				ECM_InfoUpdate(0, 0, 0, 0);
			}
			printf("\r\nPASS\r\n");
			// Communication has been initiated
			if(pRet->Head.u8Status & ECM_STA_ECAT_CONFIG_MASK){
				printf("Initialization state : ");
				ECM_EcatStateSet(0xFF, EC_STATE_NONE);
				printf("DONE\r\n");
			}
			// FIFO has been initiated
			if(pRet->Head.u8Status & ECM_STA_FIFO_EN_MASK){
				printf("Disable FIFO\r\n");
				ECM_EnableFIFO(0);
				ECM_ClearFIFO(0);
			}
			return u8Version;
		}
	}
	return 0;
}
int ECM_GetFirmwareVersion(uint8_t *pVersion)
{
	int nret=0;
	pCmd->Head.u8Cmd = ECM_CMD_FW_VERSION_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Ctrl = 0;
	nret=EcmCmdTransceiver(1000);
	*pVersion = pRet->Head.u8Return;
	return nret;
}
/*
 * Return
 * 0	: Idle
 * 1	: Busy
 * -1	: Error
 * */
int ECM_IsAsyncBusy(void)
{
	if(ECM_InfoUpdate(0,0,0,0)){
		if(g_u8EcmStatus & ECM_STA_ASYNC_OP_BUSY_MASK){
			return 1;
		}else{
			return 0;
		}
	}
	return -1;
}
int ECM_WaitAsyncDone(int nMS)
{
	int i=0;
	for(i=0;i<nMS;i++){
		if(ECM_IsAsyncBusy()==0){
			return 1;
		}else{
			UserDelay(1000);//1ms
		}
	}
	return 0;
}
int ECM_SetSpiDataSize(uint16_t u16SpiDataSize)
{
	int nOrgSpiPackSize;
	SPI_CMD_HEADER	*pSpiPackHead;
	uint32_t		*pSpiPackCrc;
	uint32_t		*pSpiPackStopWord;
	nOrgSpiPackSize = EcmSpiPackSizeCal(1024);
	if(nOrgSpiPackSize == 0){
		return 0;
	}
	pSpiPackHead = (SPI_CMD_HEADER *)u8TxBuf;
	pSpiPackHead->u8Cmd = ECM_CMD_SPI_RECONFIG_OP;
	pSpiPackHead->u16Size = u16SpiDataSize;
	pSpiPackHead->u8Idx++;
	pSpiPackHead->u32StartWord = ECM_START_WORD;
	pSpiPackCrc = (uint32_t *)&u8TxBuf[nOrgSpiPackSize-8];
	pSpiPackStopWord = (uint32_t *)&u8TxBuf[nOrgSpiPackSize-4];
	*pSpiPackCrc = ECM_CRC_MAGIC_NUM;
	*pSpiPackStopWord = ECM_STOP_WORD;
	UserSpiDataExchange(u8TxBuf, u8RxBuf, nOrgSpiPackSize);
	UserDelay(100000);//100ms
	return 1;
}

int ECM_GetSpiDataSize(uint16_t *pu16SpiDataSize)
{
	pCmd->Head.u8Cmd = ECM_CMD_SPI_PACK_SIZE_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	if(EcmCmdTransceiver(1000)){
		memcpy(pu16SpiDataSize , pRet->Data, 2);
		return 1;
	}
	return 0;
}

int ECM_SetTxFIFOCnt(uint8_t u8TxCnt)
{
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_SET_TX_CNT;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8TxCnt;
	return EcmCmdTransceiver(1000);
}
int ECM_SetRxFIFOCnt(uint8_t u8RxCnt)
{
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_SET_RX_CNT;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8RxCnt;
	return EcmCmdTransceiver(1000);
}

int ECM_GetTxFIFOCnt(uint8_t *pu8Cnt)
{
	int nret;
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_GET_TX_CNT;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret = EcmCmdTransceiver(1000);
	*pu8Cnt = pRet->Head.u8Return;
	return nret;
}
int ECM_GetRxFIFOCnt(uint8_t *pu8Cnt)
{
	int nret;
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_GET_RX_CNT;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret = EcmCmdTransceiver(1000);
	*pu8Cnt = pRet->Head.u8Return;
	return nret;
}

int ECM_EnableFIFO(uint8_t u8Enable)
{
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_ENABLE;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8Enable;
	return EcmCmdTransceiver(1000);
}

int ECM_ClearFIFO(uint8_t u8TxRx)
{
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_CLR_OP;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8TxRx;
	return EcmCmdTransceiver(1000);
}
int ECM_InitFIFO(void)
{
	int nret=0;
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_INIT;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret=EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	return nret;
}

int ECM_EcatInit(uint16_t DCActCode, uint32_t CycTime, int32_t CycShift, int nDisableFIFO)
{
	EC_DCSYNC_H *pDcSyncCmd = (EC_DCSYNC_H *)pCmd->Data;
	pDcSyncCmd->Slave = ECM_INDEX;
	pDcSyncCmd->Mode = (DCActCode)&0xff;
	pDcSyncCmd->Act = (DCActCode>>8)&0xff;
	pDcSyncCmd->CyclTime0 = CycTime;
	pDcSyncCmd->CyclShift = CycShift;
	pDcSyncCmd->Paddle = 0x81;//Disable FIFO first
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_INIT_OP;
	pCmd->Head.u16Size = sizeof(EC_DCSYNC_H);
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Ctrl |= ECM_CTL_CLR_ALL_ERR_MASK;
	return EcmCmdTransceiver_WaitAyncDone(10000,0,0);
}
int ECM_EcatReconfig()
{
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_RECONFIG_OP;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver_WaitAyncDone(10000,0,0);
}

int8_t ECM_EcatSlvCntGet()
{
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_SLV_CNT_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	if(EcmCmdTransceiver(1000)==1){
		return pRet->Head.u8Return;
	}
	return -1;
}

int ECM_EcatStateSet(uint8_t u8Slave, uint8_t u8State)
{
	int nret=0;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_STATE_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8Slave;
	pCmd->Head.u8Data[0] = u8State;
	nret=EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	return nret;
}
int ECM_EcatStateGet(uint8_t u8Slave, uint8_t *pu8State)
{
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_STATE_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8Slave;
	if(EcmCmdTransceiver(1000) == 1){
		*pu8State = pRet->Head.u8Return;
		return 1;
	}
	return 0;
}
int ECM_EcatStateCheck(uint8_t u8Slave, uint8_t u8State)
{
	int nret=0;
	int8_t n8Return;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_STATE_CHECK;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = u8Slave;
	pCmd->Head.u8Data[0] = u8State;
	nret=EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	return nret;
}
int ECM_StateCheck(uint8_t u8Slave, uint8_t u8ExpectState, int TimeOutMS)
{
	uint8_t u8State;
	int i=0;
	if(ECM_EcatStateSet(u8Slave, u8ExpectState)){
		for(i=0;i<TimeOutMS;i++){
			UserDelay(1000);
			if(ECM_EcatStateCheck(u8Slave, u8ExpectState)){
                int ret = ECM_EcatStateGet(u8Slave, &u8State);
				if(ret > 0 && u8State == u8ExpectState){
					return 1;
				}
			}
		}
	}
	return 0;
}
int ECM_EcatPdoConfigSet( uint8_t Slave, PDO_CONFIG_HEAD *pConfigData)
{
	int nret;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_CONFIG_SET;
	pCmd->Head.u16Size = sizeof(PDO_CONFIG_HEAD);
	pCmd->Head.u8Idx++;
	pConfigData->Slave = Slave;
	memcpy(pCmd->Data, pConfigData, sizeof(PDO_CONFIG_HEAD));
	nret = EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	if(nret == -2){
		printf("Get SDO abort code\r\n");
	}
	return nret;
}
int ECM_EcatPdoConfigReq(uint8_t Slave, uint16_t SmaIdx)
{
	int nret;
    PDO_CONFIG_HEAD *pTxCmd = (PDO_CONFIG_HEAD *)pCmd->Data;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_CONFIG_REQ;
	pCmd->Head.u16Size = sizeof(PDO_CONFIG_HEAD);
	pCmd->Head.u8Idx++;
	pTxCmd->Slave = Slave;
	pTxCmd->SmaIdx = SmaIdx;
	nret = EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	return nret;
}
int ECM_EcatPdoConfigGet(PDO_CONFIG_HEAD *pBuf)
{
	int nret;
	uint8_t IdxCheck;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_CONFIG_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret = EcmCmdTransceiver(1000);
	if(nret == 1){
		memcpy(pBuf,pRet->Data,pRet->Head.u16Size);
	}
	return nret;
}

int ECM_EcatSdoReq(uint8_t OP, \
		uint8_t Slave, \
		uint16_t Index, \
		uint8_t SubIndex, \
		uint16_t size, \
		int Timeout, \
		uint8_t *Data)
{
	int nret;
	SDO_CMD_HEAD *pSdoCmd = (SDO_CMD_HEAD *)pCmd->Data;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_SDO_REQ;
	pCmd->Head.u8Idx++;
	if(OP == ECM_SDO_OP_WR){
		pCmd->Head.u16Size = 12+size;
		memcpy(pSdoCmd->Data,Data,size);
	}else{
		pCmd->Head.u16Size = 12;
	}
	//pCmd->Head.u8Ctrl = 0;
	pSdoCmd->OP = OP;
	pSdoCmd->Slave = Slave;
	pSdoCmd->Index = Index;
	pSdoCmd->SubIndex = SubIndex;
	pSdoCmd->size = size;
	pSdoCmd->Timeout = Timeout;
	pSdoCmd->CA = 0;
	nret = EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	if(nret == -2){
		printf("Get SDO abort code\r\n");
	}
	return nret;
}

int ECM_EcatSdoCAReq(uint8_t OP, \
		uint8_t Slave, \
		uint16_t Index, \
		uint8_t SubIndex, \
		uint16_t size, \
		int Timeout, \
		uint8_t *Data)
{
	int nret;
	SDO_CMD_HEAD *pSdoCmd = (SDO_CMD_HEAD *)pCmd->Data;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_SDO_REQ;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Ctrl = 0;
	pSdoCmd->OP = OP;
	pSdoCmd->Slave = Slave;
	pSdoCmd->Index = Index;
	pSdoCmd->SubIndex = SubIndex;
	pSdoCmd->size = size;
	pSdoCmd->Timeout = Timeout;
	pSdoCmd->CA = 1;
	if(OP == ECM_SDO_OP_WR){
		pCmd->Head.u16Size = 12+size;
		memcpy(pSdoCmd->Data,Data,size);
	}else{
		pCmd->Head.u16Size = 12;
	}
	nret = EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	if(nret == -2){
		printf("Get SDO abort code\r\n");
	}
	return nret;
}

int ECM_EcatSdoGet(uint8_t *pBuf)
{
	uint16_t u16Size;
	if(EcmUniversalCmd(ECM_CMD_ECAT_SDO_GET,0,0,0,0,0,NULL,NULL,NULL,&u16Size,pBuf)==1){
		return u16Size;
	}
	return 0;
}

int ECM_EcatSdoWrite(uint8_t Slave, \
		uint16_t Index, \
		uint8_t SubIndex, \
		uint16_t size, \
		int Timeout, \
		uint8_t *Data)
{
	return ECM_EcatSdoReq( ECM_SDO_OP_WR, Slave, Index, SubIndex, size, Timeout, Data);
}

int ECM_EcatSdoRead(uint8_t Slave, \
		uint16_t Index, \
		uint8_t SubIndex, \
		uint16_t size, \
		int Timeout, \
		uint8_t *Data)
{
	int nret = ECM_EcatSdoReq( ECM_SDO_OP_RD, Slave, Index, SubIndex, size, Timeout, Data);
	if(nret <= 0){
		printf("ECM_EcatSdoReq READ error\n");
		return nret;
	}
	return ECM_EcatSdoGet(Data);
}
int ECM_Drv402SM_AdvConfig(uint8_t TblIdx, uint8_t SlvIdx, uint8_t ContrlWordOffset, uint8_t StateWordOffset)
{
	pCmd->Head.u8Cmd = ECM_CMD_402_CONFIG_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = TblIdx;
	pCmd->Head.u8Data[0] = ContrlWordOffset;
	pCmd->Head.u8Data[1] = StateWordOffset;
	pCmd->Head.u8Data[2] = (CIA402_FSM_CTL_ENABLE_MASK | CIA402_FSM_CTL_FAULT_AUTORST_MASK);
	pCmd->Head.u8Data[3] = SlvIdx;
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver(1000);
}
int ECM_Drv402SM_Enable(uint8_t TblIdx)
{
	pCmd->Head.u8Cmd = ECM_CMD_402_CTL_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = TblIdx;
	pCmd->Head.u8Data[0] = (CIA402_FSM_CTL_ENABLE_MASK | CIA402_FSM_CTL_FAULT_AUTORST_MASK);
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver(1000);
}
int ECM_Drv402SM_Disable(uint8_t TblIdx)
{
	pCmd->Head.u8Cmd = ECM_CMD_402_CTL_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = TblIdx;
	pCmd->Head.u8Data[0] = 0;
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver(1000);
}
int ECM_Drv402SM_StateSet(uint8_t TblIdx, uint8_t State)
{
	pCmd->Head.u8Cmd = ECM_CMD_402_STATE_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = TblIdx;
	pCmd->Head.u8Data[0] = State;
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver(1000);
}
int ECM_Drv402SM_StateGet(uint8_t TblIdx, uint8_t *pState)
{
	int i;
	uint8_t IdxCheck;
	pCmd->Head.u8Cmd = ECM_CMD_402_STATE_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = TblIdx;
	pCmd->Head.u8Idx++;
	i = EcmCmdTransceiver(1000);
	if(i == 1){
		*pState = pRet->Head.u8Return;
		return i;
	}
	return i;
}
int ECM_Drv402SM_StateCheck(uint8_t TblIdx, uint8_t ExceptState, int TimeOutMS)
{
	int i;
	uint8_t State;
	i=ECM_Drv402SM_StateSet(TblIdx, ExceptState);
	if(i<0){
		return i;
	}
	if(ECM_Drv402SM_StateGet(TblIdx, &State) == 1){
		if((State & CIA402_SW_STATE_MASK) == ExceptState){
			return 1;
		}
	}
	printf("(%d) 0x%X 0x%X\r\n",TblIdx,State,ExceptState);
	return 0;
}
int ECM_FifoRxPdoSizeGet()
{
	int i;
	uint16_t PdoSize;
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_PACK_SIZE_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = 0;	//	0:	RX
							//	1:	TX
	i = EcmCmdTransceiver(1000);
	if(i == 1){
		memcpy(&PdoSize, pRet->Data, 2);
		return PdoSize;
	}
	return 0;
}
int ECM_FifoTxPdoSizeGet()
{
	int i;
	uint16_t PdoSize;
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_PACK_SIZE_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Param = 1;	//	0:	RX
							//	1:	TX
	i = EcmCmdTransceiver(1000);
	if(i == 1){
		memcpy(&PdoSize, pRet->Data, 2);
		return PdoSize;
	}
	return 0;
}
uint8_t ECM_EcatPdoDataExchange(uint8_t u8OP, uint8_t *pRxData, uint8_t *pTxData, uint16_t *pu16DataSize)
{
	uint8_t IdxCheck;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_DATA_OP;
	pCmd->Head.u16Size = *pu16DataSize;
	pCmd->Head.u8Idx++;
	pCmd->Head.u8Data[0] = u8OP;
	if((u8OP & ECM_PDO_WR_OP) == ECM_PDO_WR_OP){
		memcpy(pCmd->Data, pRxData, pCmd->Head.u16Size);
	}
	if(SpiDataExchange(&IdxCheck, 0)) {
		if (pRet->Head.u8Cmd == ECM_CMD_ECAT_PDO_DATA_OP) {
			if ((pRet->Head.u8Return & ECM_PDO_RD_OP) == ECM_PDO_RD_OP) {
				memcpy(pTxData, pRet->Data, pRet->Head.u16Size);
				*pu16DataSize = pRet->Head.u16Size;
			}
			return pRet->Head.u8Return;
		}else{
			return ECM_PDO_WR_OP;
		}
	}
	return 0x80;
}
int ECM_EcatPdoFifoIsFull(uint8_t u8FifoThreshold)
{
	// Notice : FIFO count update has two times delay
	if(pRet->Head.u8RxFifoCnt >= u8FifoThreshold-2){
		return 1;// FIFO count threshold reached
	}else{
		return 0;
	}
}
int ECM_EcatPdoFifoDataExchange(uint8_t u8FifoThreshold, uint8_t *pRxData, uint8_t *pTxData, uint16_t u16DataSize, uint8_t *pu8RxPdoFifoCnt, uint8_t *CrcErrCnt, uint8_t *WkcErrCnt)
{
	// Notice : FIFO count update has two times delay
	if(pRet->Head.u8RxFifoCnt >= u8FifoThreshold-2){
		return -2;// FIFO count threshold reached
	}
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_DATA_FIFO_OP;
	pCmd->Head.u16Size = u16DataSize;
	pCmd->Head.u8Param = 1;
	pCmd->Head.u8Data[0] = (ECM_FIFO_WR | ECM_FIFO_RD);
	pCmd->Head.u8Idx++;
	memcpy(pCmd->Data, pRxData, pCmd->Head.u16Size);
	if(SpiDataExchange(0,0) == 0){
		printf("CRC error\r\n");
		return -1;//CRC error
	}
	if(pu8RxPdoFifoCnt)
		*pu8RxPdoFifoCnt=pRet->Head.u8RxFifoCnt;
	if(CrcErrCnt)
		*CrcErrCnt = pRet->Head.u8CrcErrCnt;
	if(WkcErrCnt)
		*WkcErrCnt = pRet->Head.u8WkcErrCnt;
	if(pRet->Head.u8Cmd == ECM_CMD_ECAT_PDO_DATA_FIFO_OP){
		if(pRet->Head.u8Return & ECM_FIFO_RD){
			if(pRet->Head.u16Size){
				memcpy(pTxData, pRet->Data, pRet->Head.u16Size);
			}else{
				printf("zero size\r\n");
				return -4;
			}
			return pRet->Head.u16Size;
		}
		if(pRet->Head.u8Return & 0x80){
			printf("FifoPdo return 0x%X\r\n",pRet->Head.u8Return);
			return 0;
		}
	}else{
		return -3;
	}
	return -6;
}
int ECM_EcatPdoFifoDataExchangeAdv(uint8_t u8Cnt,uint8_t u8FifoThreshold, uint8_t *pRxData, uint8_t *pTxData, uint16_t u16DataSize, uint8_t *pu8RxPdoFifoCnt, uint8_t *CrcErrCnt, uint8_t *WkcErrCnt)
{
	// Notice : FIFO count update has two times delay
	if(pRet->Head.u8RxFifoCnt > u8FifoThreshold-(2*u8Cnt)){
		printf("ECM_EcatPdoFifoDataExchangeAdv error\r\n");
		return -2;// FIFO count threshold reached
	}
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_PDO_DATA_FIFO_OP;
	pCmd->Head.u16Size = u8Cnt*u16DataSize;
	pCmd->Head.u8Param = u8Cnt;
	pCmd->Head.u8Data[0] = (ECM_FIFO_WR | ECM_FIFO_RD);
	pCmd->Head.u8Idx++;
	memcpy(pCmd->Data, pRxData, pCmd->Head.u16Size);
	if(SpiDataExchange(0,0) == 0){
		printf("CRC error\r\n");
		return -1;//CRC error
	}
	if(pu8RxPdoFifoCnt)
		*pu8RxPdoFifoCnt=pRet->Head.u8RxFifoCnt;
	if(CrcErrCnt)
		*CrcErrCnt = pRet->Head.u8CrcErrCnt;
	if(WkcErrCnt)
		*WkcErrCnt = pRet->Head.u8WkcErrCnt;

	if(pRet->Head.u8Cmd == ECM_CMD_ECAT_PDO_DATA_FIFO_OP){
		if(pRet->Head.u8Return & ECM_FIFO_RD){
			if(pRet->Head.u16Size){
				memcpy(pTxData, pRet->Data, pRet->Head.u16Size);
			}else{
				printf("zero size\r\n");
				return -4;
			}
			return pRet->Head.u16Size;
		}
		if(!(pRet->Head.u8Return & ECM_FIFO_WR)){
			printf("FIFO full\r\n");
		}
		if(pRet->Head.u8Return & 0x80){
			printf("FifoPdo return 0x%X\r\n",pRet->Head.u8Return);
			return 0;
		}
	}
	return 0;
}
int ECM_EcatEepromReq(
		uint16_t OP, \
		uint16_t slave, \
		uint16_t eeproma, \
		uint16_t data, \
		uint32_t timeout)
{
	ECM_EEPROM_REQ_T *pEepromReq = (ECM_EEPROM_REQ_T *)pCmd->Data;
	pCmd->Head.u8Cmd = ECM_EEPROM_REQ;
	pCmd->Head.u8Idx++;
	pEepromReq->OP = OP;
	pEepromReq->slave = slave;
	pEepromReq->eeproma = eeproma;
	pEepromReq->data = data;
	pEepromReq->timeout = timeout;
	pCmd->Head.u16Size = sizeof(ECM_EEPROM_REQ_T);
	return SpiDataExchange(0,0);
}
int ECM_EcatEepromGet(uint64_t *pu64Data)
{
	int nret;
	if(ECM_WaitAsyncDone(1000) == 0)
			return 0;
	pCmd->Head.u8Cmd = ECM_EEPROM_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;
	nret = EcmCmdTransceiver_WaitAyncDone(1000,0,0);
	if(nret == -2){
		printf("Get SDO abort code\r\n");
	}
	return nret;
}
int ECM_GetPDOConfig(int Slave, int SmaIdx, PDO_CONFIG_HEAD *pPdoConfigBuf)
{
	int nret = ECM_EcatPdoConfigReq(Slave, SmaIdx);
	if(nret <= 0){
		return 0;
	}
	nret = ECM_EcatPdoConfigGet(pPdoConfigBuf);
	if(nret <= 0){
		return 0;
	}
	return 1;
}
int ECM_ShowPDOConfig(int Slave, int SmaIdx)
{
	int i=0,j=0;
	PDO_CONFIG_HEAD PdoConfigBuf;
	int nret = ECM_GetPDOConfig(Slave, SmaIdx, &PdoConfigBuf);
	if(nret <= 0){
		return 0;
	}
	printf("(%d) 0x%X : \r\n",Slave, SmaIdx);
	for(i=0;i<PdoConfigBuf.PDOCnt;i++){
		printf("PDO%d - MapIdx(0x%X)\r\n", i, PdoConfigBuf.MapIdx[i]);
		for(j=0; j<PdoConfigBuf.ObjsCnt[i]; j++){
			printf("\t0x%X\r\n", PdoConfigBuf.Table[i][j]);
		}
	}
	return 1;
}

int ECM_EcatDatagramReq(
        ecm_datagram_commad_t cmd,
		uint16_t position,
		uint16_t offset,
        uint32_t logicalAddress,
		uint16_t length,
		int Timeout,
		uint8_t *Data)
{
	EC_RAW_API_PARAM_H *pDatagramCmd = (EC_RAW_API_PARAM_H *)pCmd->Data;
	pCmd->Head.u8Cmd = ECM_CMD_RAW_ECAT_FUNC_REQ;
	pCmd->Head.u8Idx++;
    pCmd->Head.u16Size = sizeof(EC_RAW_API_PARAM_H) - 256 + length;
	pDatagramCmd->OP = cmd;
	pDatagramCmd->ADO = offset;
	pDatagramCmd->ADP = position;
	pDatagramCmd->LogAdr = logicalAddress;
	pDatagramCmd->length = length;
	pDatagramCmd->timeout = Timeout;
    memcpy(pDatagramCmd->data,Data,length);
	return EcmCmdTransceiver_WaitAyncDone(1000,0,0);
}

int ECM_EcatDatagramGet(uint8_t *pBuf)
{
	int nret;
	pCmd->Head.u8Cmd = ECM_CMD_RAW_ECAT_FUNC_GET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Idx++;

	nret = EcmCmdTransceiver(1000);
	if(nret == 1){
		memcpy(pBuf, pRet->Data, pRet->Head.u16Size);
		return nret;
	}
	return nret;
}

int ECM_SlaveInfoGet(uint8_t slave, uint8_t info, uint8_t *pBuf)
{
	int nret;
	pCmd->Head.u8Cmd = ECM_CMD_ECAT_SLV_INFO_GET;
    pCmd->Head.u8Param = slave;
	pCmd->Head.u16Size = 0;
    pCmd->Head.u8Data[0] = info;
	pCmd->Head.u8Idx++;
	nret = EcmCmdTransceiver(1000);
	if(nret == 1){
		memcpy(pBuf, pRet->Data, pRet->Head.u16Size);
		return nret;
	}
	return nret;
}


int ECM_SetFIFOExtDataSize(uint8_t nByteSize)
{
	pCmd->Head.u8Cmd = ECM_CMD_FIFO_EXPDO_DATA_CNT_SET;
	pCmd->Head.u16Size = 0;
	pCmd->Head.u8Param = nByteSize;
	pCmd->Head.u8Idx++;
	return EcmCmdTransceiver(1000);
}


int ECM_SetEcatIntEnable(uint32_t u32mask, uint8_t u8ActiveHigh)
{
        pCmd->Head.u8Cmd = ECM_CMD_ECAT_INT_SET_ENABLE;
        pCmd->Head.u16Size = 4;
        pCmd->Head.u8Idx++;
        pCmd->Head.u8Param = u8ActiveHigh;
        memcpy( pCmd->Data, &u32mask, 4);
        return EcmCmdTransceiver(1000);
}

int ECM_FifoClear(void){
        pCmd->Head.u8Cmd = ECM_CMD_FIFO_CLR_OP;
        pCmd->Head.u16Size = 0;
        pCmd->Head.u8Idx++;
        return EcmCmdTransceiver(1000);

}

//	(C) COPYRIGHT 2020 NEXTW Technology Corp.
